# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Forms (GUI).
#
# Authors:
# Andre Heinecke <andre.heinecke@intevation.de>
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

from pylons.i18n.translation import _, ungettext
from pylons import tmpl_context as c

import formencode

import mapwocgui.lib.helpers as h
from mapwocgui.lib.validators import QuickScanFormURLValidator
from mapwocgui.lib.validators import HCSelectionValidator
from mapwocgui.lib.validators import RessourceCheckValidator

# TODO While it would be nice to validate this, the master will complain
#      if a scan is running, anyways. It brings some moral complications to
#      import the masterproxy here. So following function is broken (and
#      uncommented where it should be called).
def check_running_scan(self, values, c):
    c.current_scan_name = master.get_state_set().get('scanname')
    if c.current_scan_name:
        h.flash(
            _(u"Quickscan can not be started while %s is still running.") % \
                c.current_scan_name)
        raise formencode.validators.Invalid(self.message("no_message", c),
               values, c)

class QuickScanForm(formencode.Schema):
    """Schema for the form to create and start the QuickScan."""

    messages = {'no_message' : " "}
    allow_extra_fields = True
    filter_extra_fields = False
    quickurl = formencode.All(formencode.validators.String(not_empty=True),
                              QuickScanFormURLValidator())

    # TODO Validate VM.
    def validate_python(self, values, c):
        # TODO see above for check_running_scan
        #check_running_scan(self, values, c)
        pass

class TimedScanForm(formencode.Schema):
    """Schema for the form to create and start the TimedScan."""

    messages = {'no_message' : " "}
    allow_extra_fields = True
    filter_extra_fields = False
    time_frame = formencode.validators.Int(not_empty=True, min=1, max=3600)

    def validate_python(self, values, c):
        """Check that there is mimimum one HC selected."""
        hckeys = [key for key in values.keys() if key.startswith('hc') and values[key] == '1']
        if len(hckeys) < 1:
            h.flash(_(u"At least one HoneyClient should be selected for timed scan!"))
            raise formencode.validators.Invalid(self.message("no_message", c),values, c)

class ScanForm(formencode.Schema):
    """Schema for the form to create a Scan."""

    chained_validators = [HCSelectionValidator, RessourceCheckValidator]
    allow_extra_fields = True
    filter_extra_fields = False
    name = formencode.All(formencode.validators.PlainText(),
                          formencode.validators.String(not_empty=True))
    description = formencode.validators.String()
    comment = formencode.validators.String(not_empty=True)
    active_window = formencode.validators.Int(not_empty=True, min=1, max=600)
    urllist = formencode.validators.NotEmpty()
    messages = {'no_message' : " "}

    def validate_python(self, values, c):
        if values["name"] == "QuickScan":
            h.flash(_(u"Name QuickScan is reserved"))
            raise formencode.validators.Invalid(self.message("no_message", c),
                   values, c)


