# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Validators for Forms (GUI).
#
# Authors:
# Andre Heinecke <andre.heinecke@intevation.de>
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

from pylons.i18n.translation import _, ungettext
from pylons import tmpl_context as c

import formencode

from mapwoc.lib.url import URLList, URL, create_URL_List
from mapwoc.lib.scanconfiguration import TransportableSC

import mapwocgui.lib.helpers as h
from mapwocgui.model.meta import Session, Base, metadata
import mapwocgui.model.masterproxy as master
from mapwocgui.lib.form_transformers import scanconfig_from_form

class HCSelectionValidator(formencode.validators.FancyValidator):
    """ Validates the HoneyClient list. """

    messages = {'empty' : _(u"You must at least select one HoneyClient."),
                'negativeNumber' : _(u"Cannot start a negative number of HoneyClients."),
                'noNode' : _(u"One ore more HoneyClients do"
                              " not appear to be on any Node."),
                'resourceCheckFail' : _(u"This configuration did not pass"
                              " the ressource check on one or more nodes."),
                'no_message' : " "}

    def validate_python(self, values, c):
        atleastone = False
        visitors = {}

        hckeys = [key for key in values.keys() if key.startswith('hc')]
        for key in hckeys:
            if int(values[key]) < 0:
                h.flash(self.message('negativeNumber' ,c ))
                raise formencode.validators.Invalid(
                        self.message("no_message", c), values, c)
            if int(values[key]):
                atleastone = True
            if len(key.split(",")) < 2:
                h.flash(self.message("noNode", c))
                raise formencode.validators.Invalid(
                        self.message("no_message", c), values, c)

        if not atleastone:
            # Workaround to make the error message look consistent
            h.flash(self.message("empty", c))
            raise formencode.validators.Invalid(self.message("no_message", c),
                    values, c)

class RessourceCheckValidator(formencode.validators.FancyValidator):
    """
    Ask the master to asks its node whether they have the ressources to start
    this scan.
    """

    messages = {
                'no_message' : " ",
                'not_enough_ressources' : _(u"Ressource Check Failed:"
                     " One or more Nodes do not have sufficient ressources.")
               }

    def validate_python(self, values, c):
        sc = scanconfig_from_form(values)

        # The URL-Lists from this scanconfig are created from a form
        # and not read from database, thus not connected to db-Session.
        # Establish this connection now. 
        Session.add(sc)
        dburllists = []
        for ul in sc.urlcollection:
            dbul = Session.query(URLList).filter_by(name=ul.name).first()
            dburllists.append(dbul)
        sc.urlcollection = dburllists

        tr = master.check_scan(TransportableSC(sc))
        Session.rollback()
        if not tr.error:
            pass
        else:
            h.flash(self.message('not_enough_ressources', c) + tr.message)
            raise formencode.validators.Invalid(self.message("no_message", c),
                                                values, c)


class QuickScanFormURLValidator(formencode.validators.FancyValidator):
    messages = {'no_message' : " "}

    def validate_python(self, values, c):
        urllist,invalids = create_URL_List("tmp",
                                    values, None)
        for invalid in invalids:
            h.flash(_(u"Invalid URL: %s") % invalid)
        if invalids:
            raise formencode.validators.Invalid(self.message("no_message", c),
                   values, c)
        return values

